import overseer
from overseer.api.v1 import api
from flask import request


@api.route("/status", methods=["GET"])
def status():
    """
    GET:
        REQUEST: /api/v1/status
        RESPONSE:
            {
                version: "0.0.1",,
                active_scans: [
                    <WEBSOCKET_DATA>
                ],
            }
    """
    return {
        "version": overseer.__version__,
        "active_scans": overseer.scan_manager.get_status(),
    }


@api.route("/scans", methods=["POST"])
def scans_post():
    """
    POST:
        REQUEST: /api/v1/scans
        DATA: { target: 'www.google.com' }

        RESPONSE: { status: "started" }
    """
    data = request.get_json()
    if data is None or "target" not in data:
        return {"error": "Missing 'target'"}, 422

    history_id = overseer.scan_manager.perform_scan(data["target"])
    if history_id is None:
        return {"error": "Unable to resolve hostname."}, 422

    return {"status": "started", "id": history_id}


@api.route("/scans/<string:target>", methods=["GET"])
def scan_get_history(target):
    """
    GET:
        REQUEST: /api/v1/scans/www.google.com
                 /api/v1/scans/1.1.1.1
        RESPONSE: { "data": [ <ARRAY_OF_SCAN_HISTORY> ] }
    """
    page = 1

    if overseer.scan_manager.is_ip(target):
        scan_results = overseer.database.get_scan_results_by_target(
            page, ip_addr=target
        )
    else:
        scan_results = overseer.database.get_scan_results_by_target(
            page, hostname=target
        )

    return {"data": __normalize_scan_results(scan_results)}


# TODO
@api.route("/search", methods=["GET"])
def scan_get_all(search):
    """
    GET:
        REQUEST: /api/v1/search?query=1.1.1.1
                 /api/v1/search?query=192.168.0.0/24
                 /api/v1/search?query=www.google.com&page=2
        RESPONSE: { data: [ <ARRAY_OF_TARGETS> ] }
    """
    return "SEARCH PLACEHOLDER"


def __normalize_scan_results(scan_results):
    return list(
        map(
            lambda x: {
                "results": x.results.split(","),
                "created_at": x.created_at,
                "status": x.status,
                "error": x.error,
            },
            scan_results,
        )
    )
