from sqlalchemy import Column, Integer, String, ForeignKey, DateTime
from sqlalchemy.orm import declarative_base, relationship

Base = declarative_base()


class ScanTarget(Base):
    __tablename__ = "scan_target"

    # Unique ID
    id = Column(Integer, primary_key=True, unique=True)

    # Integer representation of an IP Address
    ip = Column(Integer, index=True, unique=True)

    # Corresponding hostname
    hostname = Column(String, index=True, unique=True)

    # Updated At
    updated_at = Column(DateTime())

    def __repr__(self):
        return (
            f"ScanTarget(id={self.id!r}, ip={self.ip!r}, "
            f"hostname={self.hostname!r}, updated_at={self.updated_at!r})"
        )


class ScanHistory(Base):
    __tablename__ = "scan_history"

    # Unique ID
    id = Column(Integer, primary_key=True, unique=True)

    # Scan Target Reference
    target_id = Column(Integer, ForeignKey("scan_target.id"))

    # Results (53 TCP, 53 UDP)
    results = Column(String)

    # Created At
    created_at = Column(DateTime())

    # Status (IN_PROGRESS, COMPLETE, FAILED)
    status = Column(String)

    # Error Message (If applicable)
    error = Column(String)

    # Relationship
    target = relationship("ScanTarget", foreign_keys=[target_id])

    def __repr__(self):
        return (
            f"ScanHistory(id={self.id!r}, target_id={self.target_id!r}, "
            f"results={self.results!r}, created_at={self.created_at!r}, "
            f"status={self.status!r}, error={self.error!r})"
        )
